@@ -22,6 +22,7 @@ from octavia.common import constants
 import octavia.common.context
 from octavia.common import data_models
 from octavia.common import exceptions
+from octavia.db import repositories
 from octavia.tests.functional.api.v2 import base
 
 
@@ -53,6 +54,7 @@ class TestHealthMonitor(base.BaseAPITest):
         self.pool_with_listener_id = (
             self.pool_with_listener.get('pool').get('id'))
         self.set_lb_status(self.lb_id)
+        self.pool_repo = repositories.PoolRepository()
         self._setup_udp_lb_resources()
 
     def _setup_udp_lb_resources(self):
@@ -81,7 +83,7 @@ class TestHealthMonitor(base.BaseAPITest):
     def test_get(self):
         api_hm = self.create_health_monitor(
             self.pool_id, constants.HEALTH_MONITOR_HTTP,
-            1, 1, 1, 1).get(self.root_tag)
+            1, 1, 1, 1, tags=['test_tag']).get(self.root_tag)
         # Set status to ACTIVE/ONLINE because set_lb_status did it in the db
         api_hm['provisioning_status'] = constants.ACTIVE
         api_hm['operating_status'] = constants.ONLINE
@@ -168,12 +170,13 @@ class TestHealthMonitor(base.BaseAPITest):
     def test_get_all(self):
         api_hm = self.create_health_monitor(
             self.pool_id, constants.HEALTH_MONITOR_HTTP,
-            1, 1, 1, 1).get(self.root_tag)
+            1, 1, 1, 1, tags=['test_tag']).get(self.root_tag)
         self.set_lb_status(self.lb_id)
         hms = self.get(self.HMS_PATH).json.get(self.root_tag_list)
         self.assertIsInstance(hms, list)
         self.assertEqual(1, len(hms))
         self.assertEqual(api_hm.get('id'), hms[0].get('id'))
+        self.assertEqual(api_hm['tags'], hms[0]['tags'])
 
     def test_get_all_not_authorized(self):
         self.create_health_monitor(
@@ -610,6 +613,108 @@ class TestHealthMonitor(base.BaseAPITest):
         self.assertEqual(hm1['id'],
                          hms['healthmonitors'][0]['id'])
 
+    def test_get_all_tags_filter(self):
+        pool1 = self.create_pool(
+            self.lb_id,
+            constants.PROTOCOL_HTTP,
+            constants.LB_ALGORITHM_ROUND_ROBIN,
+            name='pool1').get('pool')
+        self.set_lb_status(self.lb_id)
+        pool2 = self.create_pool(
+            self.lb_id,
+            constants.PROTOCOL_HTTP,
+            constants.LB_ALGORITHM_ROUND_ROBIN,
+            name='pool2').get('pool')
+        self.set_lb_status(self.lb_id)
+        pool3 = self.create_pool(
+            self.lb_id,
+            constants.PROTOCOL_HTTP,
+            constants.LB_ALGORITHM_ROUND_ROBIN,
+            name='pool3').get('pool')
+        self.set_lb_status(self.lb_id)
+        hm1 = self.create_health_monitor(
+            pool1.get('id'), constants.HEALTH_MONITOR_HTTP,
+            1, 1, 1, 1, tags=['test_tag1', 'test_tag2']).get(self.root_tag)
+        self.set_lb_status(self.lb_id)
+        hm2 = self.create_health_monitor(
+            pool2.get('id'), constants.HEALTH_MONITOR_PING,
+            1, 1, 1, 1, tags=['test_tag2', 'test_tag3']).get(self.root_tag)
+        self.set_lb_status(self.lb_id)
+        hm3 = self.create_health_monitor(
+            pool3.get('id'), constants.HEALTH_MONITOR_TCP,
+            1, 1, 1, 1, tags=['test_tag4', 'test_tag5']).get(self.root_tag)
+        self.set_lb_status(self.lb_id)
+
+        hms = self.get(
+            self.HMS_PATH,
+            params={'tags': 'test_tag2'}
+        ).json.get(self.root_tag_list)
+        self.assertIsInstance(hms, list)
+        self.assertEqual(2, len(hms))
+        self.assertEqual(
+            [hm1.get('id'), hm2.get('id')],
+            [hm.get('id') for hm in hms]
+        )
+
+        hms = self.get(
+            self.HMS_PATH,
+            params={'tags': ['test_tag2', 'test_tag3']}
+        ).json.get(self.root_tag_list)
+        self.assertIsInstance(hms, list)
+        self.assertEqual(1, len(hms))
+        self.assertEqual(
+            [hm2.get('id')],
+            [hm.get('id') for hm in hms]
+        )
+
+        hms = self.get(
+            self.HMS_PATH,
+            params={'tags-any': 'test_tag2'}
+        ).json.get(self.root_tag_list)
+        self.assertIsInstance(hms, list)
+        self.assertEqual(2, len(hms))
+        self.assertEqual(
+            [hm1.get('id'), hm2.get('id')],
+            [hm.get('id') for hm in hms]
+        )
+
+        hms = self.get(
+            self.HMS_PATH,
+            params={'not-tags': 'test_tag2'}
+        ).json.get(self.root_tag_list)
+        self.assertIsInstance(hms, list)
+        self.assertEqual(1, len(hms))
+        self.assertEqual(
+            [hm3.get('id')],
+            [hm.get('id') for hm in hms]
+        )
+
+        hms = self.get(
+            self.HMS_PATH,
+            params={'not-tags-any': ['test_tag2', 'test_tag4']}
+        ).json.get(self.root_tag_list)
+        self.assertIsInstance(hms, list)
+        self.assertEqual(0, len(hms))
+
+        hms = self.get(
+            self.HMS_PATH,
+            params={'tags': 'test_tag2',
+                    'tags-any': ['test_tag1', 'test_tag3']}
+        ).json.get(self.root_tag_list)
+        self.assertIsInstance(hms, list)
+        self.assertEqual(2, len(hms))
+        self.assertEqual(
+            [hm1.get('id'), hm2.get('id')],
+            [hm.get('id') for hm in hms]
+        )
+
+        hms = self.get(
+            self.HMS_PATH,
+            params={'tags': 'test_tag2', 'not-tags': 'test_tag2'}
+        ).json.get(self.root_tag_list)
+        self.assertIsInstance(hms, list)
+        self.assertEqual(0, len(hms))
+
     def test_empty_get_all(self):
         response = self.get(self.HMS_PATH).json.get(self.root_tag_list)
         self.assertIsInstance(response, list)
@@ -680,6 +785,22 @@ class TestHealthMonitor(base.BaseAPITest):
         self.assertEqual('/', api_hm.get('url_path'))
         self.assertEqual('200', api_hm.get('expected_codes'))
 
+    def test_create_https_full(self):
+        api_hm = self.create_health_monitor(
+            self.pool_id, constants.HEALTH_MONITOR_HTTPS,
+            1, 1, 1, 1, admin_state_up=False, expected_codes='200',
+            http_method='GET', name='Test HM', url_path='/').get(self.root_tag)
+        self.assertEqual(constants.HEALTH_MONITOR_HTTPS, api_hm.get('type'))
+        self.assertEqual(1, api_hm.get('delay'))
+        self.assertEqual(1, api_hm.get('timeout'))
+        self.assertEqual(1, api_hm.get('max_retries_down'))
+        self.assertEqual(1, api_hm.get('max_retries'))
+        self.assertFalse(api_hm.get('admin_state_up'))
+        self.assertEqual('Test HM', api_hm.get('name'))
+        self.assertEqual('GET', api_hm.get('http_method'))
+        self.assertEqual('/', api_hm.get('url_path'))
+        self.assertEqual('200', api_hm.get('expected_codes'))
+
     def test_create_udp_case(self):
         api_hm = self.create_health_monitor(
             self.udp_pool_with_listener_id,
@@ -880,6 +1001,24 @@ class TestHealthMonitor(base.BaseAPITest):
         self.conf.config(group='api_settings', auth_strategy=auth_strategy)
         self.assertEqual(self.NOT_AUTHORIZED_BODY, api_hm)
 
+    def test_create_pool_in_error(self):
+        project_id = uuidutils.generate_uuid()
+        lb1 = self.create_load_balancer(uuidutils.generate_uuid(), name='lb1',
+                                        project_id=project_id)
+        lb1_id = lb1.get('loadbalancer').get('id')
+        self.set_lb_status(lb1_id)
+        pool1 = self.create_pool(
+            lb1_id, constants.PROTOCOL_HTTP,
+            constants.LB_ALGORITHM_ROUND_ROBIN).get('pool')
+        pool1_id = pool1.get('id')
+        self.set_lb_status(lb1_id)
+        self.set_object_status(self.pool_repo, pool1_id,
+                               provisioning_status=constants.ERROR)
+        api_hm = self.create_health_monitor(
+            pool1_id, constants.HEALTH_MONITOR_HTTP, 1, 1, 1, 1, status=409)
+        ref_msg = 'Pool %s is immutable and cannot be updated.' % pool1_id
+        self.assertEqual(ref_msg, api_hm.get('faultstring'))
+
     def test_create_with_listener(self):
         api_hm = self.create_health_monitor(
             self.pool_with_listener_id, constants.HEALTH_MONITOR_HTTP,
@@ -920,6 +1059,42 @@ class TestHealthMonitor(base.BaseAPITest):
             project_id=pid).get(self.root_tag)
         self.assertEqual(self.project_id, api_hm.get('project_id'))
 
+    def test_create_with_default_http_version(self):
+        # Use the default HTTP/1.0
+        api_hm = self.create_health_monitor(
+            self.pool_id, constants.HEALTH_MONITOR_HTTP,
+            1, 1, 1, 1, admin_state_up=False, expected_codes='200',
+            http_method='GET', name='Test HM', url_path='/',
+            http_version='1.0').get(self.root_tag)
+        self.assertEqual(1.0, api_hm.get('http_version'))
+
+    def test_create_without_http_version(self):
+        # Check the default http_version is 1.0
+        api_hm = self.create_health_monitor(
+            self.pool_id, constants.HEALTH_MONITOR_HTTP,
+            1, 1, 1, 1, admin_state_up=False, expected_codes='200',
+            http_method='GET', name='Test HM', url_path='/').get(self.root_tag)
+        self.assertIsNone(api_hm.get('http_version'))
+
+    def test_create_with_http_version_11_and_domain_name(self):
+        # Create with http_version 1.1 and domain_name
+        api_hm = self.create_health_monitor(
+            self.pool_id, constants.HEALTH_MONITOR_HTTPS,
+            1, 1, 1, 1, admin_state_up=False, expected_codes='200',
+            http_method='GET', name='Test HM', url_path='/',
+            http_version=1.1, domain_name='testlab.com').get(self.root_tag)
+        self.assertEqual(1.1, api_hm.get('http_version'))
+        self.assertEqual('testlab.com', api_hm.get('domain_name'))
+
+    def test_create_with_http_version_11(self):
+        # Create with http_version 1.1
+        api_hm = self.create_health_monitor(
+            self.pool_id, constants.HEALTH_MONITOR_HTTPS,
+            1, 1, 1, 1, admin_state_up=False, expected_codes='200',
+            http_method='GET', name='Test HM', url_path='/',
+            http_version=1.1).get(self.root_tag)
+        self.assertEqual(1.1, api_hm.get('http_version'))
+
     def test_bad_create(self):
         hm_json = {'name': 'test1', 'pool_id': self.pool_id}
         self.post(self.HMS_PATH, self._build_body(hm_json), status=400)
@@ -1093,12 +1268,47 @@ class TestHealthMonitor(base.BaseAPITest):
               'max_retries': 1}
         self.post(self.HMS_PATH, self._build_body(hm), status=403)
 
+    def test_bad_create_with_http_version_and_domain_name_cases(self):
+        hm_json = {'pool_id': self.pool_id,
+                   'type': constants.HEALTH_MONITOR_HTTP,
+                   'delay': 1,
+                   'timeout': 1,
+                   'max_retries_down': 1,
+                   'max_retries': 1,
+                   'expected_codes': '200',
+                   'http_version': 1.00, 'domain_name': 'testlab.com'}
+        api_hm = self.post(
+            self.HMS_PATH, self._build_body(hm_json), status=400).json
+        expect_error_msg = ("http_version 1.0 is not a valid option for "
+                            "health monitors HTTP 1.1 domain name health "
+                            "check")
+        self.assertEqual(expect_error_msg, api_hm['faultstring'])
+        for bad_case in [{'http_version': 1.0, 'domain_name': '^testla&b.com'},
+                         {'http_version': 1.1,
+                          'domain_name': 'testla\nb.com'}]:
+            hm_json = {'pool_id': self.pool_id,
+                       'type': constants.HEALTH_MONITOR_HTTP,
+                       'delay': 1,
+                       'timeout': 1,
+                       'max_retries_down': 1,
+                       'max_retries': 1,
+                       'expected_codes': '200'}
+            hm_json.update(bad_case)
+            api_hm = self.post(
+                self.HMS_PATH, self._build_body(hm_json), status=400).json
+            expect_error_msg = (
+                "Invalid input for field/attribute domain_name. Value: '%s'. "
+                "Value should match the pattern %s") % (bad_case[
+                    'domain_name'], constants.DOMAIN_NAME_REGEX)
+            self.assertEqual(expect_error_msg, api_hm['faultstring'])
+
     def test_update(self):
         api_hm = self.create_health_monitor(
             self.pool_with_listener_id,
-            constants.HEALTH_MONITOR_HTTP, 1, 1, 1, 1).get(self.root_tag)
+            constants.HEALTH_MONITOR_HTTP, 1, 1, 1, 1,
+            tags=['old_tag']).get(self.root_tag)
         self.set_lb_status(self.lb_id)
-        new_hm = {'max_retries': 2}
+        new_hm = {'max_retries': 2, 'tags': ['new_tag']}
         self.put(
             self.HM_PATH.format(healthmonitor_id=api_hm.get('id')),
             self._build_body(new_hm))
@@ -1112,6 +1322,52 @@ class TestHealthMonitor(base.BaseAPITest):
         response = self.get(self.HM_PATH.format(
             healthmonitor_id=api_hm.get('id'))).json.get(self.root_tag)
         self.assertEqual(2, response[constants.MAX_RETRIES])
+        self.assertEqual(['new_tag'], response['tags'])
+
+    def test_update_HTTPS(self):
+        api_hm = self.create_health_monitor(
+            self.pool_with_listener_id,
+            constants.HEALTH_MONITOR_HTTPS, 1, 1, 1, 1,
+            admin_state_up=False, expected_codes='200',
+            http_method='GET', name='Test HM', url_path='/').get(self.root_tag)
+        self.set_lb_status(self.lb_id)
+        self.assertEqual('/', api_hm[constants.URL_PATH])
+        new_hm = {constants.URL_PATH: '/health'}
+        self.put(
+            self.HM_PATH.format(healthmonitor_id=api_hm.get('id')),
+            self._build_body(new_hm))
+        self.assert_correct_status(
+            lb_id=self.lb_id, listener_id=self.listener_id,
+            pool_id=self.pool_with_listener_id, hm_id=api_hm.get('id'),
+            lb_prov_status=constants.PENDING_UPDATE,
+            listener_prov_status=constants.PENDING_UPDATE,
+            pool_prov_status=constants.PENDING_UPDATE,
+            hm_prov_status=constants.PENDING_UPDATE)
+        response = self.get(self.HM_PATH.format(
+            healthmonitor_id=api_hm.get('id'))).json.get(self.root_tag)
+        self.assertEqual('/health', response[constants.URL_PATH])
+
+    def test_update_http_version_and_domain_name(self):
+        api_hm = self.create_health_monitor(
+            self.pool_with_listener_id, constants.HEALTH_MONITOR_HTTP,
+            1, 1, 1, 1, admin_state_up=False, expected_codes='200',
+            http_method='GET', name='Test HM', url_path='/').get(self.root_tag)
+        self.set_lb_status(self.lb_id)
+        new_hm = {'http_version': 1.1, 'domain_name': 'testlab.com'}
+        self.put(
+            self.HM_PATH.format(healthmonitor_id=api_hm.get('id')),
+            self._build_body(new_hm))
+        self.assert_correct_status(
+            lb_id=self.lb_id, listener_id=self.listener_id,
+            pool_id=self.pool_with_listener_id, hm_id=api_hm.get('id'),
+            lb_prov_status=constants.PENDING_UPDATE,
+            listener_prov_status=constants.PENDING_UPDATE,
+            pool_prov_status=constants.PENDING_UPDATE,
+            hm_prov_status=constants.PENDING_UPDATE)
+        response = self.get(self.HM_PATH.format(
+            healthmonitor_id=api_hm.get('id'))).json.get(self.root_tag)
+        self.assertEqual(1.1, response['http_version'])
+        self.assertEqual('testlab.com', response['domain_name'])
 
     def test_update_TCP(self):
         api_hm = self.create_health_monitor(
@@ -1263,6 +1519,42 @@ class TestHealthMonitor(base.BaseAPITest):
         self.put(self.HM_PATH.format(healthmonitor_id=api_hm.get('id')),
                  self._build_body(new_hm), status=400)
 
+    def test_update_delay_none(self):
+        api_hm = self.create_health_monitor(self.pool_with_listener_id,
+                                            constants.HEALTH_MONITOR_HTTP,
+                                            1, 1, 1, 1).get(self.root_tag)
+        new_hm = {constants.DELAY: None}
+        self.set_lb_status(self.lb_id)
+        expect_error_msg = ("None is not a valid option for %s" %
+                            constants.DELAY)
+        res = self.put(self.HM_PATH.format(healthmonitor_id=api_hm.get('id')),
+                       self._build_body(new_hm), status=400)
+        self.assertEqual(expect_error_msg, res.json['faultstring'])
+
+    def test_update_max_retries_none(self):
+        api_hm = self.create_health_monitor(self.pool_with_listener_id,
+                                            constants.HEALTH_MONITOR_HTTP,
+                                            1, 1, 1, 1).get(self.root_tag)
+        new_hm = {constants.MAX_RETRIES: None}
+        self.set_lb_status(self.lb_id)
+        expect_error_msg = ("None is not a valid option for %s" %
+                            constants.MAX_RETRIES)
+        res = self.put(self.HM_PATH.format(healthmonitor_id=api_hm.get('id')),
+                       self._build_body(new_hm), status=400)
+        self.assertEqual(expect_error_msg, res.json['faultstring'])
+
+    def test_update_timeout_none(self):
+        api_hm = self.create_health_monitor(self.pool_with_listener_id,
+                                            constants.HEALTH_MONITOR_HTTP,
+                                            1, 1, 1, 1).get(self.root_tag)
+        new_hm = {constants.TIMEOUT: None}
+        self.set_lb_status(self.lb_id)
+        expect_error_msg = ("None is not a valid option for %s" %
+                            constants.TIMEOUT)
+        res = self.put(self.HM_PATH.format(healthmonitor_id=api_hm.get('id')),
+                       self._build_body(new_hm), status=400)
+        self.assertEqual(expect_error_msg, res.json['faultstring'])
+
     @mock.patch('octavia.api.drivers.utils.call_provider')
     def test_update_with_bad_provider(self, mock_provider):
         api_hm = self.create_health_monitor(
@@ -1350,6 +1642,56 @@ class TestHealthMonitor(base.BaseAPITest):
         self.assertEqual(constants.HEALTH_MONITOR_DEFAULT_EXPECTED_CODES,
                          response['expected_codes'])
 
+    def test_bad_update_http_version_and_domain_name(self):
+        api_hm = self.create_health_monitor(
+            self.pool_id, constants.HEALTH_MONITOR_HTTP,
+            1, 1, 1, 1, admin_state_up=False, expected_codes='200',
+            http_method='GET', name='Test HM', url_path='/').get(self.root_tag)
+        self.set_lb_status(self.lb_id)
+        new_hm = {'http_version': 1.0, 'domain_name': 'testlab.com'}
+        response = self.put(
+            self.HM_PATH.format(healthmonitor_id=api_hm.get('id')),
+            self._build_body(new_hm), status=400)
+        expect_error_msg = ("http_version 1.0 is not a valid option for "
+                            "health monitors HTTP 1.1 domain name health "
+                            "check")
+        self.assertEqual(expect_error_msg, response.json['faultstring'])
+
+        new_hm = {'http_version': 1.0, 'domain_name': '^testla&b.com'}
+        response = self.put(
+            self.HM_PATH.format(healthmonitor_id=api_hm.get('id')),
+            self._build_body(new_hm), status=400)
+        expect_error_msg = (
+            "Invalid input for field/attribute domain_name. Value: '%s'. "
+            "Value should match the pattern %s") % (new_hm[
+                'domain_name'], constants.DOMAIN_NAME_REGEX)
+        self.assertEqual(expect_error_msg, response.json['faultstring'])
+
+    def test_update_unset_defaults(self):
+        api_hm = self.create_health_monitor(
+            self.pool_with_listener_id, constants.HEALTH_MONITOR_HTTP,
+            1, 1, 1, 1, name='test', domain_name='test.example.com',
+            expected_codes='400', http_method='HEAD', http_version='1.1',
+            url_path='/test').get(self.root_tag)
+        new_hm = {constants.DOMAIN_NAME: None, constants.EXPECTED_CODES: None,
+                  constants.HTTP_METHOD: None, constants.HTTP_VERSION: None,
+                  constants.MAX_RETRIES_DOWN: None, 'name': None,
+                  constants.URL_PATH: None}
+        self.set_lb_status(self.lb_id)
+        res = self.put(self.HM_PATH.format(healthmonitor_id=api_hm.get('id')),
+                       self._build_body(new_hm)).json.get(self.root_tag)
+        self.assertIsNone(res[constants.DOMAIN_NAME])
+        self.assertEqual(constants.HEALTH_MONITOR_DEFAULT_EXPECTED_CODES,
+                         res[constants.EXPECTED_CODES])
+        self.assertEqual(constants.HEALTH_MONITOR_HTTP_DEFAULT_METHOD,
+                         res[constants.HTTP_METHOD])
+        self.assertIsNone(res[constants.HTTP_VERSION])
+        self.assertEqual(constants.DEFAULT_MAX_RETRIES_DOWN,
+                         res[constants.MAX_RETRIES_DOWN])
+        self.assertEqual('', res['name'])
+        self.assertEqual(constants.HEALTH_MONITOR_DEFAULT_URL_PATH,
+                         res[constants.URL_PATH])
+
     def test_delete(self):
         api_hm = self.create_health_monitor(
             self.pool_with_listener_id,
